package com.fjhx.rfid.r2000.tcp.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.fjhx.config.RFIDConfig;
import com.fjhx.constant.RFIDConstant;
import com.fjhx.constant.RedisKeyConstant;
import com.fjhx.entity.rfid.RFIDClient;
import com.fjhx.rabbitmq.enums.MsgSourceEnum;
import com.fjhx.rabbitmq.service.RabbitmqService;
import com.fjhx.redis.RedisCache;
import com.fjhx.rfid.r2000.tcp.callback.TcpCallBackAllData;
import com.fjhx.rfid.r2000.tcp.callback.TcpCallBackGoHeavyData;
import com.fjhx.rfid.r2000.tcp.service.TcpServiceV2;
import com.fjhx.thread.ThreadPoolManager;
import com.fjhx.utils.AjaxResult;
import com.fjhx.utils.id.IdUtils;
import com.fjhx.web.exception.BaseException;
import com.rfid.uhf.controller.impl.ReaderR2000;
import com.rfid.uhf.service.ReaderR2000Service;
import com.rfid.uhf.service.impl.ReaderR2000ServiceImpl;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.collections4.MapUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.CountDownLatch;
import java.util.stream.Collectors;

/**
 * @Description:
 * @ClassName: TcpServiceImpl
 * @Author: linqt
 * @Date: 2022/3/17 10:32
 * @Version: 1.0
 */
@Service
public class TcpServiceImplV2 implements TcpServiceV2 {

    @Autowired
    private RedisCache redisCache;

    @Autowired
    private RabbitmqService rabbitmqService;

    /**
     * 初始化实例集合
     */
    private static ConcurrentHashMap<String, ConcurrentHashMap<String, Object>> serverMap = new ConcurrentHashMap<>();

    /**
     * 启动并返回所有数据
     *
     * @param ips     读写器ip集合
     * @param busUuid 业务唯一标识
     * @param userId  业务服务端用户userId
     * @return
     */
    @Override
    public AjaxResult openAndReturnAllData(List<String> ips, String busUuid, String userId) {
        Integer count = 0;
        try {
            CountDownLatch countDownLatch = new CountDownLatch(ips.size());
            System.err.println(serverMap);
            // 启动扫描线程
            for (String ip : ips) {
                count++;
                Integer finalCount = count;
                CountDownLatch errCountDownLatch = countDownLatch;
                List<JSONObject> list = new ArrayList<>();
                ThreadPoolManager.execute(() -> {
                    try {
                        // redis tcp数据缓存key
                        String dataRedisKey = MessageFormat.format(RedisKeyConstant.RFID_TCP_DATA_REDIS_CACHE_PREFIX, ip + "-" + finalCount);
                        ReaderR2000Service service = new ReaderR2000ServiceImpl();
                        ReaderR2000 reader = service.connect(ip, RFIDConfig.getTcp().getPort(), new TcpCallBackAllData(dataRedisKey, userId, ip, list));

                        // 实例存储
                        ConcurrentHashMap<String, Object> map = new ConcurrentHashMap<>();
                        map.put("service", service);
                        map.put("reader", reader);
                        serverMap.put(ip + "-" + busUuid + "-" + finalCount, map);
                        // 开始扫描
                        service.beginInvV2(reader);

                        while (true) {
                            //休眠1s进行数据收集
                            Thread.sleep(1000);

                            if (CollectionUtils.isNotEmpty(list)) {
                                // -------------------- 使用rabbitmq派转消息 --------------------
                                JSONObject jsonObject = new JSONObject();
                                jsonObject.put("messageId", IdUtils.fastSimpleUUID());
                                jsonObject.put("msgSource", MsgSourceEnum.MSG_SOURCE_2.getKey());
                                jsonObject.put("userId", userId);
                                jsonObject.put("notices", list);
                                rabbitmqService.send(jsonObject);
                                System.err.println("jsonObject--：" + jsonObject.toJSONString());
                            }

                            list.clear();
                        }

                    } catch (Exception e) {
                        e.printStackTrace();
                        if (errCountDownLatch != null) {
                            errCountDownLatch.countDown();
                        }
                        throw new BaseException(MessageFormat.format(RFIDConstant.SERVER_START_ERR, ip));
                    }
                });
            }

            if (countDownLatch != null) {
                countDownLatch.await();
            }
        } catch (Exception e) {
            e.printStackTrace();
        }

        return AjaxResult.success();
    }

    /**
     * 启动并返回去重数据
     *
     * @param ips     读写器ip集合
     * @param busUuid 业务唯一标识
     * @param userId  业务服务端用户userId
     * @return
     */
    @Override
    public AjaxResult openAndReturnGoHeavyData(List<String> ips, String busUuid, String userId) {
        Integer count = 0;
        try {
            CountDownLatch countDownLatch = new CountDownLatch(ips.size());
            System.err.println(serverMap);
            // 启动扫描线程
            for (String ip : ips) {
                count++;
                Integer finalCount = count;
                CountDownLatch errCountDownLatch = countDownLatch;
                ConcurrentHashMap<String, JSONObject> map = new ConcurrentHashMap<>();
                ThreadPoolManager.execute(() -> {
                    try {
                        // redis tcp数据缓存key
                        String dataRedisKey = MessageFormat.format(RedisKeyConstant.RFID_TCP_DATA_REDIS_CACHE_PREFIX, ip + "-" + finalCount);
                        ReaderR2000Service service = new ReaderR2000ServiceImpl();
                        ReaderR2000 reader = service.connect(ip, RFIDConfig.getTcp().getPort(), new TcpCallBackGoHeavyData(dataRedisKey, userId, ip, map));

                        // 实例存储
                        ConcurrentHashMap<String, Object> sMap = new ConcurrentHashMap<>();
                        sMap.put("service", service);
                        sMap.put("reader", reader);
                        serverMap.put(ip + "-" + busUuid + "-" + finalCount, sMap);
                        // 开始扫描
                        service.beginInvV2(reader);

                        while (true) {
                            //休眠1s进行数据收集
                            Thread.sleep(1000);

                            // -------------------- 使用rabbitmq派转消息 --------------------
                            JSONObject jsonObject = new JSONObject();
                            jsonObject.put("messageId", IdUtils.fastSimpleUUID());
                            jsonObject.put("msgSource", MsgSourceEnum.MSG_SOURCE_2.getKey());
                            jsonObject.put("userId", userId);
                            jsonObject.put("notices", map.values().stream().collect(Collectors.toList()));
                            rabbitmqService.send(jsonObject);
                            System.err.println("jsonObject--：" + jsonObject.toJSONString());

                            map.clear();
                        }

                    } catch (Exception e) {
                        e.printStackTrace();
                        if (errCountDownLatch != null) {
                            errCountDownLatch.countDown();
                        }
                        throw new BaseException(MessageFormat.format(RFIDConstant.SERVER_START_ERR, ip));
                    }
                });
            }

            if (countDownLatch != null) {
                countDownLatch.await();
            }
        } catch (Exception e) {
            e.printStackTrace();
        }

        return AjaxResult.success();
    }

    /**
     * 关闭多台
     *
     * @param client
     * @return
     */
    @Override
    public AjaxResult closeMore(RFIDClient client) {
        ConcurrentHashMap<String, JSONObject> dataMap = new ConcurrentHashMap<>();
        List<JSONObject> list = new ArrayList<>();

        Integer count = 0;
        List<String> ips = RFIDConfig.getTcp().getIps();
        try {
            if (client == null) {
                throw new BaseException("参数缺失！");
            }
            if (CollectionUtils.isNotEmpty(client.getIpList())) {
                ips = client.getIpList();
            }
            CountDownLatch countDownLatch = new CountDownLatch(ips.size());
            // 关闭线程
            for (String ip : ips) {
                count++;
                Integer finalCount = count;
                CountDownLatch finalCountDownLatch = countDownLatch;
                try {
                    // redis tcp数据缓存key
                    String dataRedisKey = MessageFormat.format(RedisKeyConstant.RFID_TCP_DATA_REDIS_CACHE_PREFIX, ip + "-" + finalCount);

                    // 获取实例
                    ConcurrentHashMap<String, Object> map = serverMap.get(ip + "-" + finalCount);
                    ReaderR2000Service service = (ReaderR2000Service) map.get("service");
                    ReaderR2000 reader = (ReaderR2000) map.get("reader");

                    // 结束扫描
                    service.stopInvV2(reader);

                    // 关闭连接
                    service.disconnect(reader);

                    // 移除服务
                    serverMap.remove(ip + "-" + finalCount);

                    // 获取数据缓存
                    ConcurrentHashMap<String, JSONObject> data = redisCache.getCacheObject(dataRedisKey);
                    if (MapUtils.isEmpty(data)) {
//                        System.err.println(ip + "-" + finalCount + "当前缓存中取出的数据为空！");
                        continue;
                    }
//                    System.err.println(ip + "-" + finalCount + "当前缓存中取出的数据量：" + data.size());
                    dataMap.putAll(data);

                    // 删除数据缓存
                    redisCache.deleteObject(dataRedisKey);
                } catch (Exception e) {
                    e.printStackTrace();
                } finally {
                    if (finalCountDownLatch != null) {
                        finalCountDownLatch.countDown();
                    }
                }
            }
//            System.err.println("最终总结果数量--:" + dataMap.size());

            if (MapUtils.isNotEmpty(dataMap)) {
                list = dataMap.values().stream().collect(Collectors.toList());
            }

            if (countDownLatch != null) {
                countDownLatch.await();
            }
        } catch (Exception e) {
            e.printStackTrace();
        }

        return AjaxResult.success(RFIDConstant.READ_DATA_SUCCESS, list);
    }
}
