package com.fjhx.rfid.r2000.udp.callback;

import com.alibaba.fastjson.JSONObject;
import com.fjhx.config.RFIDConfig;
import com.fjhx.constant.RFIDConstant;
import com.fjhx.rabbitmq.enums.MsgSourceEnum;
import com.fjhx.rabbitmq.service.RabbitmqService;
import com.fjhx.redis.RedisCache;
import com.fjhx.rfid.r2000.udp.utils.Utility;
import com.fjhx.utils.StringUtils;
import com.fjhx.utils.spring.SpringUtils;
import io.netty.buffer.ByteBuf;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.SimpleChannelInboundHandler;
import io.netty.channel.socket.DatagramPacket;
import lombok.Data;
import org.apache.commons.collections4.MapUtils;

import java.io.File;
import java.io.FileWriter;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.concurrent.ConcurrentHashMap;

@Data
public class UdpCallBackSaveLog extends SimpleChannelInboundHandler<DatagramPacket> {

    /**
     * redis实例
     */
    private RedisCache redisCache = SpringUtils.getBean(RedisCache.class);

    /**
     * rabbitmq实例
     */
    private RabbitmqService rabbitmqService = SpringUtils.getBean(RabbitmqService.class);

    /**
     * rdis缓存key
     */
    private String redisKey;

    /**
     * 用户账号
     */
    private String userName;

    private ConcurrentHashMap<String, JSONObject> map = new ConcurrentHashMap<>();

    private String readTime = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(new Date());

    public UdpCallBackSaveLog(String redisKey, String userName) {
        this.redisKey = redisKey;
        this.userName = userName;
    }

    @Override
    protected void channelRead0(ChannelHandlerContext ctx, DatagramPacket msg) {
        FileWriter fwriter = null;
        try {
            ByteBuf byteBuf = msg.content();
            byte[] bytes = new byte[byteBuf.readableBytes()];
            byteBuf.readBytes(bytes);
            String str = Utility.bytes2HexString(bytes);

            // 如果返回的结果长度小于20，则跳过
            if (StringUtils.isNotBlank(str) && str.length() > 20) {
                // 截取天线号
                String antNo = str.substring(RFIDConstant.UDP_RESULT_DATA_SUBSTRING_ANT_NO_BEGIN_INDEX, RFIDConstant.UDP_RESULT_DATA_SUBSTRING_ANT_NO_END_INDEX);
                // 截取rfid
                String rfid = str.substring(RFIDConstant.UDP_RESULT_DATA_SUBSTRING_RFID_BEGIN_INDEX, str.length() - RFIDConstant.UDP_RESULT_DATA_SUBSTRING_RFID_END_INDEX);

                String content = "RFID：" + rfid + "， ANTNO: " + antNo + "， TIME: " + readTime + "\r\n";
//                System.err.println("UDP--：" + content);


                // 封装实际数据
                JSONObject object = new JSONObject();
                object.put("RFID", rfid);
                object.put("ANTNO", antNo);

                // 存在文件夹则创建
                String filePath = RFIDConfig.getLog().getSavePath() + new SimpleDateFormat("yyyyMMdd").format(new Date()) + "\\";
                File file = new File(filePath);
                if (!file.exists() && !file.isDirectory()) {
                    file.mkdir();
                }
                filePath += new SimpleDateFormat("yyyyMMdd HH").format(new Date()) + ".txt";
                // true表示不覆盖原来的内容，而是加到文件的后面。若要覆盖原来的内容，直接省略这个参数就好
                fwriter = new FileWriter(filePath, true);

                // 过滤重复写入
                if (MapUtils.isEmpty(map) || map.get(rfid) == null) {
                    fwriter.write(content);

                    if (org.apache.commons.lang3.StringUtils.isNotBlank(userName)) {
                        // -------------------- 往websocket推送消息 --------------------
                        JSONObject jsonObject = new JSONObject();
                        jsonObject.put("messageId", rfid);
                        jsonObject.put("msgSource", MsgSourceEnum.MSG_SOURCE_2.getKey());
                        jsonObject.put("userName", userName);
                        jsonObject.put("notices", object);
                        // 使用rabbitmq派转消息
                        rabbitmqService.send(jsonObject);
                    }
                }

                // 收集数据
                map.put(rfid, object);

                // 存入缓存
                redisCache.setCacheObject(redisKey, map);
            }
        } catch (Exception ex) {
            ex.printStackTrace();
        } finally {
            try {
                if (fwriter != null) {
                    fwriter.flush();
                    fwriter.close();
                }
            } catch (Exception ex) {
                ex.printStackTrace();
            }
        }
    }
}